<?php
/**
 * Emoji.
 * Note: Database must use 'utf8mb4' to allow emojis. 'utf8' is not enough.
 *
 * @package Flat101\Security\Model\Block
 */

namespace Flat101\Security\Model\Block;

use Carbon_Fields\Field\Field;
use Flat101\Security\Model\Entity\Block;
use Flat101\Security\Model\Entity\Setting;

/**
 * Emoji class.
 */
class EmojiBlock extends Block {

	/**
	 * Block constructor.
	 */
	public function __construct() {
		parent::__construct(
			esc_html__( 'Emojis', 'flat101-security' ),
			'emoji',
			array(
				new Setting( 'disabled', true, esc_html__( 'Disable Emojis', 'flat101-security' ) ),
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_description() {
		return wp_kses_post(
			__(
				'The Emojis functionality allows for the control of emoji rendering and usage within your WordPress site. When
				enabled, this feature disables the display of emojis, which can be beneficial for speeding up site load times
				and ensuring a more professional appearance on certain types of sites. Note that for emojis to work properly,
				your database must be configured to use the "utf8mb4" character set as "utf8" is insufficient. For further
				insight on emojis and character encoding in WordPress, you may refer to the
				<a href="https://codex.wordpress.org/Emoji" target="_blank">WordPress Codex</a> and
				<a href="https://make.wordpress.org/core/2015/04/02/the-utf8mb4-upgrade/" target="_blank">this article</a>
				on the utf8mb4 upgrade.',
				'flat101-security'
			)
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function get_fields() {
		return array(
			Field::make( 'checkbox', $this->get_option_name( 'disabled' ), $this->get_setting_label( 'disabled' ) )
				->set_default_value( $this->get_setting_default_value( 'disabled' ) ),
		);
	}

	// PHPDoc (inherited).
	// phpcs:ignore Squiz.Commenting.FunctionComment.Missing
	public function run() {
		if ( $this->get_setting_value( 'disabled' ) ) {
			$this->disabled();
		}
	}

	/**
	 * Disable the emoji's
	 */
	public function disabled() {
		remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
		remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
		remove_action( 'wp_print_styles', 'print_emoji_styles' );
		remove_action( 'admin_print_styles', 'print_emoji_styles' );
		remove_filter( 'the_content_feed', 'wp_staticize_emoji' );
		remove_filter( 'comment_text_rss', 'wp_staticize_emoji' );
		remove_filter( 'wp_mail', 'wp_staticize_emoji_for_email' );
		add_filter( 'tiny_mce_plugins', array( $this, 'disable_emojis_tinymce' ) );
		add_filter( 'wp_resource_hints', array( $this, 'disable_emojis_remove_dns_prefetch' ), 10, 2 );
	}

	/**
	 * Filter function used to remove the tinymce emoji plugin.
	 *
	 * @param array $plugins An array of default TinyMCE plugins.
	 * @return array Difference betwen the two arrays
	 */
	public function disable_emojis_tinymce( $plugins ) {
		if ( is_array( $plugins ) ) {
			return array_diff( $plugins, array( 'wpemoji' ) );
		} else {
			return array();
		}
	}

	/**
	 * Remove emoji CDN hostname from DNS prefetching hints.
	 *
	 * @param array  $urls URLs to print for resource hints.
	 * @param string $relation_type The relation type the URLs are printed for.
	 * @return array Difference betwen the two arrays.
	 */
	public function disable_emojis_remove_dns_prefetch( $urls, $relation_type ) {
		if ( 'dns-prefetch' === $relation_type ) {
			// This filter is documented in wp-includes/formatting.php.
			// phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedHooknameFound
			$emoji_svg_url = apply_filters( 'emoji_svg_url', 'https://s.w.org/images/core/emoji/14.0.0/svg/' );

			$urls = array_diff( $urls, array( $emoji_svg_url ) );
		}

		return $urls;
	}
}
