<?php
/**
 * A block is a class that is responsible for solving a specific security or optimization functionality.
 *
 * @package Flat101\Security\Model\Entity
 */

namespace Flat101\Security\Model\Entity;

use Carbon_Fields\Container\Theme_Options_Container;
use Carbon_Fields\Field\Field;

use const Flat101\Security\PREFIX;

/**
 * This is the parent class of all blocks in this plugin.
 */
abstract class Block {

	/**
	 * Block title. It will be displayed to the user as the header of the group of fields.
	 *
	 * @var string
	 */
	protected $title;

	/**
	 * Block key.
	 *
	 * @var string
	 */
	protected $key;

	/**
	 * Key-value associative array.
	 *
	 * @var Setting[]
	 */
	protected $settings;

	/**
	 * Show form fields?
	 *
	 * @var bool
	 */
	protected $show = true;


	/**
	 * Block constructor.
	 *
	 * @param string    $title Block title. It should already be translated.
	 * @param string    $unique_key Block key. It must be unique and will act as a prefix for all fields.
	 * @param Setting[] $default_settings Settings array.
	 */
	public function __construct( $title, $unique_key, $default_settings ) {

		// Add index for quick access.
		$settings = array();
		foreach ( $default_settings as $setting ) {
			$settings[ $setting->get_key() ] = $setting;
		}

		$this->title    = $title;
		$this->key      = $unique_key;
		$this->settings = $settings;
	}


	/**
	 * Block key.
	 *
	 * @return string
	 */
	public function get_key() {
		return $this->key;
	}


	/**
	 * Use (prefix + block key + setting key) to have a WP option unique name.
	 * Will be used as field name in forms.
	 *
	 * @param string $setting_key This key has no prefix.
	 * @return string
	 */
	public function get_option_name( $setting_key ) {
		return PREFIX . '__' . $this->key . '__' . $setting_key;
	}


	/**
	 * Return settings as WP options.
	 *
	 * @return array
	 */
	public function get_options() {
		$result = array();

		foreach ( $this->settings as $key => $setting ) {
			$result[ $this->get_option_name( $key ) ] = $setting->value;
		}

		return $result;
	}


	/**
	 * Block title. It will be displayed to the user as the header of the group of fields.
	 *
	 * @return string
	 */
	public function get_title() {
		return $this->title;
	}


	/**
	 * Block description. It will be displayed to the user below the block title.
	 * Ex:
	 *   return __( 'This is a title description', 'flat101-security' ),
	 *
	 * @return string
	 */
	public function get_description() {
		return '';
	}


	/**
	 * Get settings (keys have no prefix).
	 *
	 * @return Setting[] Key-value associative array
	 */
	public function get_settings() {
		return $this->settings;
	}


	/**
	 * Load the settings values from the database.
	 */
	public function load_settings() {
		foreach ( $this->settings as $key => $setting ) {
			$setting->value = carbon_get_theme_option( $this->get_option_name( $key ) );
		}
	}


	/**
	 * Update the settings if they exist in the received array.
	 *
	 * @param null|array $new_settings Key-value associative array.
	 * @return array Returns the updated settings
	 */
	public function update_settings( $new_settings ) {
		if ( ! empty( $new_settings ) ) {
			foreach ( $this->settings as $key => $setting ) {
				if ( array_key_exists( $key, $new_settings ) ) {
					$setting->value = $new_settings[ $key ];
				}
			}
		}

		return $this->settings;
	}


	/**
	 * Returns the fields of the block so that the user can modify the configuration from the form.
	 *
	 * @link https://docs.carbonfields.net/learn/fields/usage.html
	 *
	 * @return Field[]
	 */
	abstract public function get_fields();


	/**
	 * Default value of the form field from its key.
	 *
	 * @param string $setting_key This key has no prefix.
	 * @return string If the key does not exist, it returns null.
	 */
	public function get_setting_default_value( $setting_key ) {
		return array_key_exists( $setting_key, $this->settings ) ? $this->settings[ $setting_key ]->default_value : null;
	}


	/**
	 * Value of the form field from its key.
	 *
	 * @param string $setting_key This key has no prefix.
	 * @return string If the key does not exist, it returns null.
	 */
	public function get_setting_value( $setting_key ) {
		return array_key_exists( $setting_key, $this->settings ) ? $this->settings[ $setting_key ]->value : null;
	}


	/**
	 * Name of the form field from its key.
	 *
	 * @param string $setting_key This key has no prefix.
	 * @return string If the key does not exist, it returns the key itself.
	 */
	public function get_setting_label( $setting_key ) {
		return array_key_exists( $setting_key, $this->settings ) ? $this->settings[ $setting_key ]->label : $setting_key;
	}


	/**
	 * Add the group of fields to the form.
	 *
	 * @link https://docs.carbonfields.net/learn/containers/theme-options.html
	 *
	 * @param Theme_Options_Container $container Carbon Fields Container.
	 */
	public function add_to_container( Theme_Options_Container $container ) {
		if ( $this->show ) {
			$fields = array();

			// Show description?
			if ( ! empty( $this->get_description() ) ) {
				$fields[] = Field::make( 'html', $this->get_option_name( 'crb_html' ) )
				->set_html( sprintf( '<p>%s</p>', $this->get_description() ) );
			}

			$fields = array_merge( $fields, $this->get_fields( $container ) );

			$container->add_tab(
				$this->get_title(),
				$fields
			);
		}
	}


	/**
	 * Execute the logic of this block. The settings will already have their values updated.
	 *
	 * This method is executed in the WordPress "init" hook. If you use:
	 *     add_action( 'init', array( $this, 'my_method' ) );
	 * It will not work.
	 */
	abstract public function run();
}
